@@ -18,6 +18,10 @@
 
 ################################################
 
+import os
+import stat
+import errno
+
 from ansible import utils
 from ansible.errors import AnsibleError
 
@@ -31,5 +35,18 @@ class Connector(object):
         conn = utils.plugins.connection_loader.get(transport, self.runner, host, port, user=user, password=password, private_key_file=private_key_file)
         if conn is None:
             raise AnsibleError("unsupported connection type: %s" % transport)
+        if private_key_file:
+            # If private key is readable by user other than owner, flag an error
+            st = None
+            try:
+                st = os.stat(private_key_file)
+            except (IOError, OSError), e:
+                if e.errno != errno.ENOENT: # file is missing, might be agent
+                    raise(e)
+
+            if st is not None and st.st_mode & (stat.S_IRGRP | stat.S_IROTH):
+                raise AnsibleError("private_key_file (%s) is group-readable or world-readable and thus insecure - "
+                                   "you will probably get an SSH failure"
+                                   % (private_key_file,))
         self.active = conn.connect()
         return self.active
